@@ -1,17 +1,9 @@
 import os
-from pathlib import Path
-
-from prometheus_client import CollectorRegistry
 
 # Local tasks config
 PROMETHEUS_HOST = os.getenv("PROMETHEUS_HOST", "127.0.0.1")
 PROMETHEUS_PORT = int(os.getenv("PROMETHEUS_PORT", "8135"))
 
-registry = CollectorRegistry()
-os.environ["prometheus_multiproc_dir"] = str(Path(__file__).parent.parent.parent / "tmp/prometheus")
-if not os.path.exists(os.environ["prometheus_multiproc_dir"]):
-    os.makedirs(os.environ["prometheus_multiproc_dir"])
-
 # DB config
 # If using sqlite
 DB_FILENAME = os.getenv("DB_FILENAME", "main.db")
@@ -25,7 +17,8 @@ DB_NAME = os.getenv("DB_NAME", None)
 
 # Construct the connection string
 if DB_USER and DB_PASSWORD and DB_HOST and DB_PORT and DB_NAME:
-    DATABASE_URL = os.getenv("DATABASE_URL",
+    DATABASE_URL_ASYNC = os.getenv("DATABASE_URL",
                              f"mysql+pymysql://{DB_USER}:{DB_PASSWORD}@{DB_HOST}:{DB_PORT}/{DB_NAME}")
 else:
-    DATABASE_URL = f"sqlite+aiosqlite:///{DB_FILENAME}"
+    DATABASE_URL_ASYNC = f"sqlite+aiosqlite:///{DB_FILENAME}"
+    DATABASE_URL_SYNC = f"sqlite:///{DB_FILENAME}"
