@@ -2,16 +2,12 @@
 Network constants
 """
 
-SEED_NODES_IPV4 = [("127.0.0.1", 8888),
-                   ("127.0.0.1", 8889),
-                   ("127.0.0.1", 8890),
-                   ("127.0.0.1", 8891),
-                   ("127.0.0.1", 8892),
-                   ("fa.decentmesh.net", 8888),
-                   ("we.decentmesh.net", 8888),
+SEED_NODES_IPV4 = [("127.0.0.1", 8888, 4),
+                   ("127.0.0.1", 8889, 4),
+                   ("fa.decentmesh.net", 8888, 4),
                    ]
 
-SEED_NODES_IPV6 = []
+SEED_NODES_IPV6 = [("fe.decentmesh.net", 8888, 6)]
 
 BLOCKED_IPV4 = [
     "119.49.152.119",
