@@ -7,12 +7,11 @@ from datetime import datetime, timezone
 from socket import socket
 
 import cbor2
-import pyximport
 from _socket import gaierror
 
 from decentnet.consensus.beam2beam import HANDSHAKE_MSG
 from decentnet.consensus.beam_constants import BEAM_HASH_SIZE, BEAM_AES_ENCRYPTION_KEY_SIZE
-from decentnet.consensus.blockchain_params import BlockchainParams
+from decentnet.consensus.blockchain_params import BlockchainParams, SKIP_SIGNATURE_VERIFICATION_DEPTH
 from decentnet.consensus.byte_conversion_constants import ENDIAN_TYPE
 from decentnet.consensus.cmd_enum import CMD
 from decentnet.consensus.dev_constants import RUN_IN_DEBUG, ENCRYPTION_DEBUG
@@ -29,43 +28,35 @@ from decentnet.modules.pow.difficulty import Difficulty
 from decentnet.modules.tcp.client import TCPClient
 from decentnet.modules.transfer.packager import Packager
 
-pyximport.install()
 logger = logging.getLogger(__name__)
 
-try:
-    from decentnet.modules.cryptography.symetric_fast import AESCipher
-except ImportError as ex:
-    logger.warning("AES operating in slow mode")
-    from decentnet.modules.cryptography.symetric import AESCipher
+from decentnet.modules.cryptography.symetric import AESCipher
 
 setup_logger(RUN_IN_DEBUG, logger)
 
-pyximport.install()
-try:
-    from decentnet.modules.convert.byte_to_base64_fast import *  # noqa
-except ImportError as ex:
-    logger.debug(f"Operating in slow mode because of {ex}")
-    logger.warning("Base64 convert module is operating in slow mode")
-    from decentnet.modules.convert.byte_to_base64_slow import *
+from decentnet.modules.convert.byte_to_base64_utils import *
 
 
 class Beam:
     conn_bc: Blockchain  # This is a blockchain for connection to relay
-    comm_bc: Blockchain  # Blockchain for Beacon2Beacon communication
+    comm_bc: Blockchain | None  # Blockchain for Beacon2Beacon communication
 
-    def __init__(self, pub_key_id: int, pub_key_enc_id: int, target_key: str):
+    def __init__(self, pub_key_id: int, pub_key_enc_id: int, target_key: str, do_metrics=False):
         """
         Beam for sending data transmissions between relay and beacon, always ip_port or client_socket is required
         :param pub_key_id:
         :param target_key:
         """
+        self.do_metric = do_metrics
         self.hash = None
         self._lock = False
+        self.comm_bc = None
         self.beam_id = None
         self.alive = False
         self.connected = False
         self.pub_key = None  # Public key under which this beam is identified to others
         self.pub_key_id = pub_key_id
+        self.skip_verification = False
         _, o_pub_key = asyncio.run(KeyManager.retrieve_ssh_key_pair_from_db(
             self.pub_key_id))
 
@@ -94,7 +85,7 @@ class Beam:
         """
         if ip_port:
             try:
-                self.client = TCPClient(*ip_port)
+                self.client = TCPClient(ip_port[0], ip_port[1], ip_type=ip_port[2])
                 self.hash = Beam.create_beam_hash(self.client.port)
                 return True
             except gaierror:
@@ -121,11 +112,12 @@ class Beam:
         :param genesis_block:
         """
         self.conn_bc = Blockchain(pub_key_for_encryption=None,
-                                  beam_id=self.beam_id, name="Connection blockchain")
+                                  beam_id=self.beam_id, name="Connection blockchain",
+                                  do_metric=self.do_metric)
+        self.conn_bc.difficulty.compression_type = genesis_block.diff.compression_type
         logger.debug("Connection incoming with genesis %s" % genesis_block)
         self.__connect_incoming(genesis_block, self.target_key)
-        self.init_comm_blockchain(
-            True)  # TODO: problem its when it accepts block from redis, relay needs this
+        self.init_comm_blockchain(True)
         self._save()
         return self.connected
 
@@ -136,7 +128,8 @@ class Beam:
         """
         self.conn_bc = Blockchain(BlockchainParams.default_genesis_msg,
                                   pub_key_for_encryption=self.pub_enc_key,
-                                  beam_id=self.beam_id, name="Connection blockchain")
+                                  beam_id=self.beam_id, name="Connection blockchain",
+                                  do_metric=False)
         logger.debug(f"Connecting outgoing from {self.target_key}")
         self.__connect_outbound_connection(self.pub_key_id, self.target_key)
 
@@ -145,14 +138,16 @@ class Beam:
         self._save()
         return self.connected
 
-    def init_comm_blockchain(self, insert: bool):
+    def init_comm_blockchain(self, insert: bool, do_metric: bool = None):
         """
         Initializes communication blockchain from genesis block of connection blockchain
         """
         logger.debug(
             f"Initiation of communication blockchain with pub_enc {self.conn_bc.pub_key_for_encryption}")
         self.comm_bc = Blockchain(pub_key_for_encryption=self.conn_bc.pub_key_for_encryption,
-                                  beam_id=self.beam_id, name="Communication Blockchain")
+                                  beam_id=self.beam_id, name="Communication Blockchain",
+                                  do_metric=self.do_metric if do_metric is None else do_metric)
+        self.comm_bc.difficulty.compression_type = self.conn_bc.difficulty.compression_type
         if insert:
             if not self.comm_bc.insert(self.conn_bc.chain[0]):
                 raise Exception(
@@ -165,16 +160,16 @@ class Beam:
         self._lock = False
 
     def __connect_outbound_connection(self, pub_key_id, target_key):
-        self.alive = len(self.conn_bc.chain) > 0
+        self.alive = len(self.conn_bc) > 0
         logger.debug(
             f"Sending Genesis block of Connection and Communication BC -> \n{self.conn_bc.get_last()}")
         response_handshake = self.send_block(pub_key_id, target_key,
                                              self.conn_bc.get_last())
 
         if not response_handshake:
-            logger.warning("Got empty response")
+            logger.warning(f"Got empty response, target {target_key}")
             self.close()
-            raise Exception("Got empty response")
+            raise Exception(f"Got empty response, target {target_key}")
 
         logger.debug("Response from Relay %s" % response_handshake)
         block = Block.from_bytes(response_handshake["data"])
@@ -253,6 +248,10 @@ class Beam:
     def __connect_incoming(self, genesis_block: Block, target_key: str):
         self.alive = self.conn_bc.insert(genesis_block)
 
+        if not self.alive:
+            logger.warning("Failed to insert genesis block, closing connection.")
+            self.close()
+
         block_data = cbor2.loads(genesis_block.data)
         logger.debug("Received genesis block data: %s" % block_data)
         received_encryption_pub_key = block_data["enc_pub_key"]
@@ -297,7 +296,7 @@ class Beam:
 
     def send_block(self, pub_key_id, target_key, block: Block, request_ack=True) -> dict:
         data = Packager.pack(pub_key_id,
-                             block, target_key)
+                             block, target_key, self.skip_verification)
         logger.debug("Sending block %s, will wait for ack..." % block)
         response = self.client.send_message(data, request_ack)
         return response
@@ -334,11 +333,6 @@ class Beam:
                                  False)
         return True
 
-    def send_new_difficulty(self, diff: Difficulty):
-        self.client.send_message(diff.to_bytes(), ack=False)
-        logger.debug("New difficulty sent %s" % diff)
-        self.conn_bc.difficulty = diff
-
     def send_connection_data(self, data_raw: bytes, ack: bool = False) -> dict:
         """
         Sending connection data,
@@ -358,7 +352,7 @@ class Beam:
         :return:
         """
         logger.debug("Trying to send communication data %s B" % (len(data_raw)))
-        if len(self.comm_bc.chain) == 0:
+        if len(self.comm_bc) == 0:
             logger.debug("Inserted genesis block from connection blockchain as communication is outbound")
             self.comm_bc.insert(self.conn_bc.chain[0])
 
@@ -393,17 +387,17 @@ class Beam:
 
         if not blockchain.insert(block):
             raise Exception(f"Failed to insert block {block}")
-        packaged_data = Packager.pack(self.pub_key_id, block, self.target_key)
+        packaged_data = Packager.pack(self.pub_key_id, block, self.target_key,
+                                      self.skip_verification if self.comm_bc else False)
 
-        response = self.client.send_message(packaged_data,
-                                            ack=ack)  # TODO: Message is not delivered
+        response = self.client.send_message(packaged_data,ack=ack)
         logger.debug(f"Response from sending message {response}")
         return response
 
     def decrypt_block_data(self, block: Block) -> dict:
         return cbor2.loads(self.encryptor_relay.decrypt(block.data))
 
-    def fetch_message(self) -> tuple[Block, dict, bytes | bytearray | str]:
+    def fetch_message(self, do_metrics=None) -> tuple[Block, dict, bytes | bytearray | str]:
         """
                Waits for and processes an incoming block message from a connected client.
 
@@ -438,36 +432,39 @@ class Beam:
 
         # Ignore broadcast blocks of other that got connected
         incoming_raw = self.client.receive_message(decode=False)
-        verified, incoming_data, verified_csig = Packager.unpack(incoming_raw)
+        verified, incoming_data, verified_csig = Packager.unpack(incoming_raw, self.skip_verification)
         Packager.check_verified(incoming_data, verified)
         block = Block.from_bytes(incoming_data["data"])
 
-        #### WORKAROUJND TODO: FIX
-        if block.index == 0 and len(self.comm_bc.chain) == 1:
+        # WORKAROUND TODO: FIX
+        if block.index == 0 and len(self.comm_bc) == 1:
             # Workaround if comm blockchain is initialized with improper genesis
-            self.init_comm_blockchain(False)
+            self.init_comm_blockchain(False, do_metric=do_metrics)
             self.comm_bc.insert(block)
 
+            if not self.skip_verification:
+                self.skip_verification = self.check_if_verification_skippable()
+
         if verified_csig is not None:  # Broadcast is faulty or handled incorrectly
             # Verifying command signature
             Packager.check_verified(incoming_data, verified_csig)
             logger.debug(f"Handling connection data {incoming_data['cpub']} {CMD(incoming_data['cmd']).name}")
             self.handle_connection_data(incoming_data, block)
-            return self.fetch_message()
+            return self.fetch_message(do_metrics=do_metrics)
         elif block.index == 0:
             load_data = cbor2.loads(block.data)
             logger.debug(f"Received enc pub key genesis block from {incoming_data['pub']}")
             asyncio.run(self.save_new_pub_key(load_data["enc_pub_key"],
                                               description=f"Key from {incoming_data['pub']}",
                                               can_encrypt=True))
-            return self.fetch_message()
+            return self.fetch_message(do_metrics=do_metrics)
 
-        if incoming_data["pub"] == self.target_key and block.index < len(self.comm_bc.chain):
+        if incoming_data["pub"] == self.target_key and block.index < len(self.comm_bc):
             logger.debug(f"Obtained genesis block twice from target {self.target_key}")
             logger.debug(f"Current Genesis {self.comm_bc.chain[0]}")
             logger.debug(f"Received genesis {block}")
 
-            return self.fetch_message()
+            return self.fetch_message(do_metrics=do_metrics)
 
         if self.encryptor_beacon is None:
             logger.debug(f"Expecting handshake block, trying to process")
@@ -475,20 +472,21 @@ class Beam:
             self.comm_bc.difficulty = block.diff
             self.comm_bc.insert(
                 block)  # Genesis block differs, need to get genesis block from sender maybe broadcast ?
+            if not self.skip_verification:
+                self.skip_verification = self.check_if_verification_skippable()
             self.send_communication_data(cbor2.dumps(HANDSHAKE_MSG))
             logger.debug("Waiting for incoming traffic")
-            return self.fetch_message()
+            return self.fetch_message(do_metrics=do_metrics)
 
         logger.debug(f"Received block {block}")
 
-        # Adjust difficulty according to sender
         # TODO: add requesting difficulty to avoid spam
-        # TODO:
-        #  genesis block is different because is inserted from conn_bc,
-        #  from broadcast it can be unwrapped as broadcast block
         self.comm_bc.difficulty = block.diff
         self.comm_bc.insert(block)
 
+        if not self.skip_verification:
+            self.skip_verification = self.check_if_verification_skippable()
+
         if ENCRYPTION_DEBUG:
             try:
                 logger.debug(f"PWD {self.encryptor_beacon.password} {self.encryptor_beacon.salt}")
@@ -497,15 +495,18 @@ class Beam:
                 pass
         return block, incoming_data, incoming_raw
 
+    def check_if_verification_skippable(self):
+        return len(self.comm_bc) > SKIP_SIGNATURE_VERIFICATION_DEPTH
+
     def handle_connection_data(self, incoming_data: dict, incoming_block: Block):
         logger.debug(f"> Processing received connection block {incoming_data}")
         self.conn_bc.difficulty = incoming_block.diff
         self.conn_bc.insert(incoming_block)
 
-        if incoming_block.index == 0 and len(self.conn_bc.chain) > 0:
+        if incoming_block.index == 0 and len(self.conn_bc) > 0:
             if (cmd := incoming_data.get("cmd", None)) and cmd == CMD.BROADCAST.value:
-                self.flow_net.add_edge(incoming_data["pub"], self.target_key,
-                                       DEFAULT_CAPACITY)
+                asyncio.run(self.flow_net.add_edge(incoming_data["pub"], self.target_key,
+                                                   DEFAULT_CAPACITY))
                 logger.debug("Adding edge from broadcast block")
             else:
                 logger.debug(
