@@ -12,7 +12,12 @@ setup_logger(RUN_IN_DEBUG, logger)
 
 class ProcessingBlock:
     @staticmethod
-    def proces_broadcast_block(network: FlowNetwork, data: dict):
+    async def proces_broadcast_block(network: FlowNetwork, data: dict):
         logger.debug(
             f"Adding edge from broadcast data {data['pub']} => {data['target']}")
-        network.add_edge(data["pub"], data["target"], DEFAULT_CAPACITY)
+        await network.add_edge(data["pub"], data["target"], DEFAULT_CAPACITY)
+
+    @staticmethod
+    async def decrease_ttl_broadcast_block(data: dict):
+        data["ttl"] -= 1
+        return data
