@@ -15,7 +15,7 @@ class TestDifficultyConversion(unittest.TestCase):
         for t_cost, m_cost, p_cost, n_bits, hash_len_chars in test_cases:
             with self.subTest(t_cost=t_cost, m_cost=m_cost, p_cost=p_cost, n_bits=n_bits,
                               hash_len_chars=hash_len_chars):
-                original = Difficulty(t_cost, m_cost, p_cost, n_bits, hash_len_chars)
+                original = Difficulty(t_cost, m_cost, p_cost, n_bits, hash_len_chars, 12)
                 converted_bytes = original.to_bytes()
                 reconstructed = Difficulty.from_bytes(converted_bytes)
 
