@@ -11,7 +11,7 @@ class BlockProcessingCase(unittest.TestCase):
         port = 8888
         target_key = "Asn25jmE_EdqzpXH28-ecvGY5b05K4LF80sjtRUNjlhh"
 
-        beacon = Beacon(host, port, pub_key_id=1)
+        beacon = Beacon(host, port, pub_key_id=1, ipv=4)
         beam = beacon.create_beam(target_key, 4)
 
         for _ in range(500000):
